@@ -1,4 +1,4 @@
-# coding=UTF-8
+from __future__ import absolute_import
 # Copyright (c) 2010-2014 openpyxl
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -23,23 +23,22 @@
 # @author: see AUTHORS file
 
 
-from openpyxl.shared.xmltools import (
+from openpyxl.xml.functions import (
     Element,
     SubElement,
     get_document_content
     )
-from openpyxl.shared.ooxml import (
+from openpyxl.xml.constants import (
     CHART_NS,
     DRAWING_NS,
     REL_NS,
     PKG_REL_NS
     )
-from openpyxl.shared.compat import (
+from openpyxl.compat import (
     iteritems,
     safe_string
     )
-from openpyxl.chart import (
-    Chart,
+from openpyxl.charts import (
     ErrorBar,
     BarChart,
     LineChart,
@@ -111,7 +110,7 @@ class BaseChartWriter(object):
             SubElement(pPr, '{%s}defRPr' % DRAWING_NS)
             r = SubElement(p, '{%s}r' % DRAWING_NS)
             SubElement(r, '{%s}rPr' % DRAWING_NS, {'lang':self.chart.lang})
-            t = SubElement(r, '{%s}t' % DRAWING_NS).text = self.chart.title
+            SubElement(r, '{%s}t' % DRAWING_NS).text = self.chart.title
             SubElement(title, '{%s}layout' % CHART_NS)
 
     def _write_axis_title(self, axis, ax):
@@ -127,7 +126,7 @@ class BaseChartWriter(object):
             SubElement(pPr, '{%s}defRPr' % DRAWING_NS)
             r = SubElement(p, '{%s}r' % DRAWING_NS)
             SubElement(r, '{%s}rPr' % DRAWING_NS, {'lang':self.chart.lang})
-            t = SubElement(r, '{%s}t' % DRAWING_NS).text = axis.title
+            SubElement(r, '{%s}t' % DRAWING_NS).text = axis.title
             SubElement(title, '{%s}layout' % CHART_NS)
 
     def _write_axis(self, plot_area, axis, label):
